<?php
class AccessTest extends PHPUnit_Framework_TestCase
{
	protected $backupGlobalsBlacklist = array('master_ary', 'Supported_Object_Types');
	function setUp()
	{
		evil_global_setup();
		set_user('host_7', array('op5_admins'));
		require_once('include/import.inc.php');
		global $HOME, $IMPORTING;
		$cfgs = array($HOME.'mangleconfig.cfg', $HOME.'testconfig.cfg');
		$this->assertTrue(
			import_config('phptap', $cfgs),
			"import_config('phptap', '" . print_r($cfgs, true) . "')"
		);
		$IMPORTING = false;

		$this->assertNotSame(false, file_put_contents('custom_config.php', '<?php $DEBUG = TRUE; $HOME = "'.realpath(getenv('WORK_DIR')).'/"; $Config["main"] = $HOME."nagios.cfg"; $Config["cgi"] = $HOME."cgi.cfg";'));
	}

	function tearDown()
	{
		$this->assertTrue(unlink('custom_config.php'));
		evil_global_teardown();
	}

	# All scripts have a bad habit of running exit() when they fail.
	# Therefore, when running pages, run them in a subprocess.
	function run_page($filename, $group, $args) {
		$user = get_user();
		$res = shell_exec('php test/unit/websiterunner.php '.$filename.' '.$user.' '.$group.' '.$args);
		#print $res;
		return $res;
	}

	function testUserRightExtraction()
	{
		set_user('host_7', array('op5_admins'));
		naemon_nacoma_Model::$objects_for_user = array();
		$rights = get_rights();
		$this->assertTrue(is_array($rights), "get_rights returned array");
		$this->assertTrue(isset($rights['system_information']), "user can see system info");
		$this->assertTrue(isset($rights['configuration_information']), "user can see config info");
		$this->assertTrue(isset($rights['host_view_all']), "user can see all hosts");
		set_user('contact1', array('contact1'));
		naemon_nacoma_Model::$objects_for_user = array();
		$rights = get_rights();
		$this->assertTrue(isset($rights['system_information']), "user can see system info");
		$this->assertTrue(!isset($rights['host_view_all']), "user can't see all hosts");
		set_user('nonexisting', array('nonexisting'));
		naemon_nacoma_Model::$objects_for_user = array();
		$rights = get_rights();
		$this->assertTrue(count($rights) === 0, "No rights array for non-user");
	}

	function testValidateUserRightsHost()
	{
		set_user('contact_limited', array('contact_limited'));
		naemon_nacoma_Model::$objects_for_user = array();
		$obj = new host_nacoma_Model('host_1');
		$obj->validate_object();
		$this->assertTrue(empty($obj->validation_error_list), 'User is allowed to store his host');

		$obj->obj['host_name'] = 12;
		$obj->validate_object();
		$this->assertTrue(empty($obj->validation_error_list), 'User is allowed to store his updated host');

		$obj = new host_nacoma_Model('host_2');
		$obj->obj['host_name'] = 12;
		$this->assertTrue(!$obj->validate_object(), "User shouldn't be allowed to store others' host");
	}

	function testValidateUserRightsService()
	{
		set_user('contact_limited', array('contact_limited', 'contact_limited_add_delete'));
		naemon_nacoma_Model::$objects_for_user = array();
		$orig_obj = new host_nacoma_Model('host_1');
		$obj = $orig_obj->clone_object();
		$obj->obj['host_name'] = 'foo';
		$obj->obj['max_check_attempts'] = '1';
		$obj->obj['notification_interval'] = '1';
		$obj->obj['notification_period'] = '1';
		$obj->obj['notification_options'] = array('d');
		$obj->validate_object();
		$this->assertTrue(empty($obj->validation_error_list), 'User is allowed to add new hosts');

		$obj = new service_nacoma_Model('host_1;Domain');
		$obj->validate_object();
		$this->assertEmpty($obj->validation_error_list, 'User should be allowed to store his services, got: '.var_export($obj->validation_error_list, true));
		$obj = $obj->clone_object();
		$obj->obj['service_description'] = 'foo';
		$obj->obj['host_name'] = 3;
		$obj->validate_object();
		$this->assertTrue(empty($obj->validation_error_list), 'User is allowed to add new services');

		set_user('contact_limited', array('contact_limited'));
		naemon_nacoma_Model::$objects_for_user = array();
		$obj = new service_nacoma_Model('host_2;Domain');
		$this->assertTrue(!$obj->validate_object(), "User shouldn't be allowed to store others' service");
	}

	function testValidateUserRightsRelations()
	{
		set_user('contact_limited', array('contact_limited'));
		naemon_nacoma_Model::$objects_for_user = array();
		# The user is only allowed to modify relations between his own objects. This
		# means all other changes are reverted. In some situations, this might be
		# unexpected.
		$obj = new host_nacoma_Model('host_1');
		$oldp = $obj->obj['parents'];
		sort($oldp);
		$this->assertNotEmpty($oldp, 'Host has the parent property set');
		$obj->normalize(array('parents' => array()));
		$newp = $obj->obj['parents'];
		sort($newp);
		$this->assertEquals($oldp, $newp, "User shouldn't be able to manipulate relations to others' objects");

		$this->setExpectedException('NacomaRelationException');
		$obj->normalize(array('parents' => array(15)));
		$newp = $obj->obj['parents'];
		sort($newp);
		$this->assertEquals($oldp, $newp, 'User can\'t add relations to others\' objects');

		$otherobj = new host_nacoma_Model('host_10');
		$obj->obj['parents'] = array($otherobj->id);
		$obj->normalize($obj->denormalize());
		$this->assertEmpty($obj->validation_error_list, 'User is allowed to store his host with modified relations');
		$newp = $obj->obj['parents'];
		sort($newp);
		$correctp = $oldp;
		$correctp[] = $otherobj->id;
		sort($correctp);
		$this->assertEquals($correctp, $newp, 'User can add relations between his objects');

		set_user('host_7', array('op5_admins'));
		naemon_nacoma_Model::$objects_for_user = array();

		$obj = new timeperiod_nacoma_Model(1);
		$obj->obj['alias'] = 'foobar';
		$obj->normalize($obj->denormalize());
		$this->assertEmpty($obj->validation_error_list, "User is allowed to modify 'allow' objects");

		$obj = $obj->clone_object();
		$obj->obj['timeperiod_name'] = 'foo';
		$obj->validate_object();
		$this->assertEmpty($obj->validation_error_list, "User is allowed to create 'allow' objects");

		set_user('host_7', array('op5_admins'));
		naemon_nacoma_Model::$objects_for_user = array();

		$obj = new timeperiod_nacoma_Model(1);
		$obj->obj['alias'] = 'foobar';
		$obj->validate_object();
		$this->assertEmpty($obj->validation_error_list, "User is allowed to modify 'allow' objects");

		set_user('contact_limited', array('contact_limited'));
		naemon_nacoma_Model::$objects_for_user = array();

		$obj = $obj->clone_object();
		$obj->obj['timeperiod_name'] = 'foo';
		$obj->validate_object();
		$this->assertNotEmpty($obj->validation_error_list, "User isn't allowed to create 'allow' objects");

		$obj = new timeperiod_nacoma_Model(1);
		$obj->obj['alias'] = 'foobar';
		$obj->validate_object();
		$this->assertNotEmpty($obj->validation_error_list, "User isn't allowed to modify 'allow' objects");

		$obj = new timeperiod_nacoma_Model(1);
		$obj->obj['alias'] = 'foobar';
		$obj->validate_object();
		$this->assertNotEmpty($obj->validation_error_list, "User isn't allowed to modify 'allow' objects");

		$obj = $obj->clone_object();
		$obj->obj['timeperiod_name'] = 'foo';
		$obj->validate_object();
		$this->assertNotEmpty($obj->validation_error_list, "User isn't allowed to create 'allow' objects");

		set_user('contact2', array('contact2'));
		naemon_nacoma_Model::$objects_for_user = array();

		// TODO: Check that the rights match this test.
		$obj = new host_nacoma_Model('host_1');
		$obj->obj['parents'] = array();
		$obj->validate_object();
		$this->assertEmpty($obj->validation_error_list, 'Admin user is allowed to store others\' host with modified relations');
		$newp = $obj->obj['parents'];
		$this->assertEmpty($newp, 'Admin user can manipulate relations to others\' objects');
		revert_users_changes();
		set_user('contact_limited', array('contact_limited'));
		naemon_nacoma_Model::$objects_for_user = array();
		revert_users_changes();
	}

	function testAdditionalValidateUserRights()
	{
		// This verifies that bug #6667 is solved.
		set_user('contact_limited', array('contact_edit_view_all'));
		naemon_nacoma_Model::$objects_for_user = array();
		$obj = new host_nacoma_Model('host_1');
		$obj->validate_object();
		$this->assertTrue(empty($obj->validation_error_list), "User should be able to save his own host but wasn't");

		$obj = new host_nacoma_Model('host_2');
		$obj->obj['host_name'] = 12;
		$obj->validate_object();
		$this->assertTrue(!empty($obj->validation_error_list), 'User shouldn\'t be allowed to store others\' host but was');

		/**
		 * If one of the next two tests fail but not the previous two, add_delete gives more permissions than it should
		 * Tip! Check functions get_objects_for_user() and can_user_write().
		 */
		set_user('contact_limited', array('contact_edit_view_all_ad'));
		naemon_nacoma_Model::$objects_for_user = array();
		$obj = new host_nacoma_Model('host_1');
		$obj->validate_object();
		$this->assertTrue(empty($obj->validation_error_list), "User should be able to save his own host but wasn't");

		$obj = new host_nacoma_Model('host_2');
		$obj->obj['host_name'] = 12;
		$obj->validate_object();
		$this->assertTrue(!empty($obj->validation_error_list), 'User shouldn\'t be allowed to store others\' host but was');
		// End tests for bug #6667

		// Strange way of setting permissions on a user but hey, who are we to judge
		set_user('contact_limited', array('view_contact_edit_all'));
		naemon_nacoma_Model::$objects_for_user = array();
		$obj = new host_nacoma_Model('host_1');
		$obj->validate_object();
		$this->assertTrue(empty($obj->validation_error_list), "User should be able to save his own host but wasn't");

		// Technically this user should be able to edit another's host, however this host will not show up anywhere.
		$obj = new host_nacoma_Model('host_2');
		$obj->obj['host_name'] = 12;
		$obj->validate_object();
		$this->assertTrue(empty($obj->validation_error_list), 'User should be allowed to store others\' host but wasn\'t');
	}

	function testIsUserContact()
	{
		require_once(dirname(__FILE__).'/../../include/nacoma_access.inc.php');
		set_user('host_7', array('op5_admins'));
		naemon_nacoma_Model::$objects_for_user = array();
		$this->assertTrue(is_user_contact('host', array(), array(), false), 'Superusers have access to any host object');
		$this->assertTrue(is_user_contact('service', array(), array(), false), 'Superusers have access to any service object');
		set_user('contact1', array('contact1'));
		naemon_nacoma_Model::$objects_for_user = array();
		$this->assertTrue(!is_user_contact('host', array(), array(), false), "Limited users don't have access to any host object");
		$this->assertTrue(!is_user_contact('service', array(), array(), false), "Limited users don't have access to any service object");
		$user_obj = new contact_nacoma_Model('contact1');
		$uid = $user_obj->id;
		$this->assertTrue(is_user_contact('host', array($uid), array(), false), "Limited users that are contacts can access hosts");
		$this->assertTrue(is_user_contact('service', array($uid), array(), false), "Limited users that are contacts can access services");
		$cgroup = new contactgroup_nacoma_Model('support');
		$cgid = $cgroup->id;
		$this->assertTrue(is_user_contact('host', array(), array($cgid), false), "Limited users that are contacts can access hosts");
		$this->assertTrue(is_user_contact('service', array(), array($cgid), false), "Limited users that are contacts can access services");
	}

	function testUserCanOnlyEditService()
	{
		set_user('host_7', array('op5_admins'));
		naemon_nacoma_Model::$objects_for_user = array();
		$ug = new usergroup_nacoma_Model('services_only');
		$ug->obj['access_rights'] = array('service_add_delete', 'service_view_contact', 'service_edit_contact', 'host_view_contact', 'service_template_view_all', 'configuration_information', 'command_view_all');
		$this->assertTrue($ug->save_object(), 'New usergroup should be saved, but got: '.var_export($ug->validation_error_list, true));
		set_user('contact_limited', array('services_only'));
		naemon_nacoma_Model::$objects_for_user = array();
		$host = new host_nacoma_Model('host_10');
		# rely on PHP's ability to cast "new" to 0:
		$this->assertTrue($host->id > 0, "Load contact's host should be fine");
		$service = new service_nacoma_Model();
		$service->expand_object();
		$service->obj['host_name'] = $host->id;
		$service->obj['service_description'] = 'test_service';
		$this->setExpectedException('NacomaRelationException');
		$service->normalize(array('contacts' => array('contact_limited'), 'template' => 'default-service'));
		$service->obj['check_command'] = 1;
		$this->assertTrue($service->save_object(), 'Creating new service should work, but got: '.var_export($service->validation_error_list, true));
		$this->assertTrue(!isset($service->obj['contacts']), 'The service should have the contacts removed when we have no contact rights');
		$this->assertSame($service->obj['host_name'], $host->id, 'The service should still have the host_name when we have host view rights');
		$service->obj['notes'] = 'This is a note';
		$this->assertTrue($service->save_object(), 'Editing service should work, but got: '.var_export($service->validation_error_list, true));
		$res = $this->run_page('edit.php', 'services_only', "obj_type service obj_id ".$service->id);
		$this->assertTrue(strpos($res, 'Access denied') === false, "The web ui should not restrict access to see service on host we're authorized to see, said: ".$res);
		$this->assertTrue($service->banish_object(), 'Deleting service should work, but got: '.var_export($service->validation_error_list, true));
		$this->assertTrue(!$host->save_object(), 'Saving host we can only view should not work');

		$host = new host_nacoma_Model('host_20');
		$this->assertTrue(!$host->save_object(), "Saving host we can't access shouldn't work");
		$service = new service_nacoma_Model();
		$service->obj['host_name'] = $host->id;
		$service->obj['service_description'] = 'test_service';
		$service->normalize(array('template' => 'default-service'));
		$service->obj['check_command'] = 1;
		$this->assertTrue($service->save_object(), "Saving services on hosts we can't see shouldn't work");
	}

	function testUserRedirectViaHostgroup()
	{
		$svc = new service_nacoma_Model('host_4;metaservice');
		$this->assertTrue(!empty($svc->obj), "The service should be found");
		$this->assertEquals($svc->obj['service_description'], 'metaservice', "The requested description should be set");
		$hg = new hostgroup_nacoma_Model('hostgroup_meta');
		$this->assertEquals($svc->obj['hostgroup_name'], $hg->id, "The host should be replaced with the true hostgroup");
		set_user('host_7', array('op5_admins'));
		naemon_nacoma_Model::$objects_for_user = array();
		$res = $this->run_page('edit.php', 'op5_admins', 'obj_type service host host_4 service metaservice');
		$this->assertTrue(strpos($res, 'hostgroup_meta;metaservice') !== false, 'Service should be located on hostgroup, even indirectly');
		$this->assertTrue(strpos($res, 'This is a hostgroup service, changes will affect all hosts in the hostgroup.') !== false, 'An appropriate warning should be displayed post redirect');
	}

	function test_user_can_not_delete_without_delete()
	{
		set_user('contact_limited', array('contact_limited'));
		naemon_nacoma_Model::$objects_for_user = array();
		$host = new host_nacoma_Model('host_1');
		$this->assertTrue(!!$host->id, 'Loading host_1 with view_all should work');
		$this->assertTrue($host->save_object(), 'Saving host_1 with edit_contact should work');
		$this->assertTrue(!$host->banish_object(), 'Banishing host_1 with edit_contact should not work');

		$host = new host_nacoma_Model('host_12');
		$this->assertTrue(!!$host->id, 'Loading host_12 with view_all should work');
		$this->assertTrue(!$host->save_object(), 'Saving host_12 with edit_contact should not work');
		$this->assertTrue(!$host->banish_object(), 'Banishing host_12 with edit_contact should not work');

		set_user('host_7', array('op5_admins'));
		naemon_nacoma_Model::$objects_for_user = array();
		$ug = new usergroup_nacoma_Model('delete_limited');
		$ug->obj['access_rights'] = array('host_add_delete', 'host_view_all', 'host_edit_contact', 'service_add_delete', 'service_edit_contact', 'servicedependency_add_delete', 'servicedependency_edit_all', 'hostdependency_add_delete', 'hostdependency_edit_all');
		$this->assertTrue($ug->save_object(), 'New usergroup should save');

		set_user('contact_limited', array('delete_limited'));
		naemon_nacoma_Model::$objects_for_user = array();
		$host = new host_nacoma_Model('host_1');
		$this->assertTrue(!!$host->id, 'Loading host_1 with view_all should work');
		$this->assertTrue($host->save_object(), 'Saving host_1 with edit_contact should work');
		$obj = $host->obj;
		$this->assertTrue(
			$host->banish_object(),
			'Banishing host_1 with add_delete+edit_contact should work, got: ' .
				print_r($host->delete_message, true)
		);

		$host = new host_nacoma_Model('host_12');
		$this->assertTrue(!!$host->id, 'Loading host_12 with view_all should work');
		$this->assertTrue(!$host->save_object(), 'Saving host_12 with edit_contact should not work');
		$this->assertTrue(!$host->banish_object(), 'Banishing host_12 with add_delete+edit_contact should not work');

		revert_users_changes();

		set_user('host_7', array('op5_admins'));
		naemon_nacoma_Model::$objects_for_user = array();
		$host = new host_nacoma_Model('host_1');
	}
}
