<?php

class ChangelogTest extends PHPUnit_Framework_TestCase {
	function setUp()
	{
		evil_global_setup();
		set_user('host_7', array('op5_admins'));
		global $HOME, $IMPORTING;
		$cfgs = array($HOME.'mangleconfig.cfg', $HOME.'testconfig.cfg');
		require_once('include/import.inc.php');
		$this->assertTrue(
			import_config('phptap', $cfgs),
			"import_config('phptap', '" . print_r($cfgs, true) . "')"
		);
		$IMPORTING = false;
		$this->assertNotSame(false, file_put_contents('custom_config.php', '<?php $DEBUG = TRUE; $HOME = "'.realpath(getenv('WORK_DIR')).'/"; $Config["main"] = $HOME."nagios.cfg"; $Config["cgi"] = $HOME."cgi.cfg";'));
	}

	function tearDown()
	{
		$this->assertTrue(unlink('custom_config.php'));
		nacoma_db::exec_query('DELETE FROM changelog');
		evil_global_teardown();
	}

	function testExportOverlay()
	{
		set_user('host_7', array('op5_admins'));
		$rel = new host_nacoma_Model('host_1');
		$orig_hostname = $rel->obj['host_name'];
		$rel->obj['host_name'] = 'foobar';
		$rel->save_object();
		$this->assertEmpty($rel->validation_error_list, 'Can save object');
		set_user('contact1', array('contact1'));
		$obj = new host_nacoma_Model('hostgroup_2');
		$obj->obj['parents'] = array($rel->id);
		$obj->save_object();
		$this->assertEmpty($obj->validation_error_list, 'Can save object');
		$changes = new ChangeLog_nacoma_Model();
		$overlay = null; $_ = null;
		$changes->undo_others_changes($overlay, $_);
		$old_obj = $overlay['host'][$obj->id];
		$old_rel = $overlay['host'][$rel->id];
		$this->assertTrue(in_array($rel->id, $old_obj->obj['parents']), "User's changes doesn't revert when undoing others'");
		$this->assertEquals($old_rel->obj['host_name'], $orig_hostname, "Other user's changes reverts when being undone");
		$new_obj = new host_nacoma_Model($obj->id);
		$new_rel = new host_nacoma_Model($rel->id);
		$this->assertTrue(in_array($rel->id, $new_obj->obj['parents']), "Changes stays in DB");
		$this->assertEquals($new_rel->obj['host_name'], 'foobar', "Changes stays in DB");
		revert_users_changes();
		set_user('host_7', array('op5_admins'));
		revert_users_changes();
	}

	function testChangedFilesIsSet()
	{
		set_user('contact1', array('contact1'));
		$obj = new host_nacoma_Model('host_1');
		$obj->obj['alias'] = 'temp alias';
		$obj->save_object();
		$changes = new ChangeLog_nacoma_Model();
		$_ = null; $files = null;
		$changes->undo_others_changes($_, $files);
		$this->assertTrue(!empty($files), "There exists a file with changes when there are changes");
		revert_users_changes();
	}


	function testExportOverlayResurrectsMyDeleted()
	{
		set_user('contact1', array('contact1'));
		$rel = new host_nacoma_Model('host_1');
		$obj = new host_nacoma_Model('hostgroup_2');
		$obj->obj['parents'] = array($rel->id);
		$obj->save_object();
		$this->assertEmpty($obj->validation_error_list, 'User can add parents');
		set_user('host_7', array('op5_admins'));
		$obj_hostname = $obj->obj['host_name'];
		$obj->banish_object();
		set_user('contact1', array('contact1'));
		$changes = new ChangeLog_nacoma_Model();
		$overlay = null; $_ = null;
		$changes->undo_others_changes($overlay, $_);
		$old_obj = $overlay['host'][$obj->id];
		$this->assertEquals($old_obj->obj['host_name'], $obj_hostname, "After undoing others' changes, object resurrects");
		$this->assertTrue(in_array($rel->id, $old_obj->obj['parents']), "After undoing others' changes, other pending objects have references to previously deleted object");
		$new_rel = new host_nacoma_Model('host_1');
		$new_obj = new host_nacoma_Model('hostgroup_2');
		$this->assertEquals($new_obj->obj['host_name'], null, "Deleted and resurrected object stays dead in DB");
	}

	function testExportOverlayDoesntKillMyNew()
	{
		set_user('host_7', array('op5_admins'));

		set_user('contact_limited', array('test_admins'));

		$obj = new host_nacoma_Model();
		$obj->get_default_object();
		$obj->obj['host_name'] = 'temp object';
		$obj->obj['alias'] = 'temp object';
		$obj->obj['address'] = 'no address';
		$obj->id = 'new';
		$obj->save_object();
		$this->assertEmpty($obj->validation_error_list, 'User can save new host');

		$changes = new ChangeLog_nacoma_Model();
		$overlay = null; $_ = null;
		$changes->undo_others_changes($overlay, $_);
		// Technically, not having it in the overlay at all would be just as
		// valid, as long as it isn't set to false.
		$this->assertTrue(!empty($overlay['host'][$obj->id]), 'Newly created host is set in overlay');
		$this->assertEquals($overlay['host'][$obj->id]->obj['alias'], 'temp object', 'Newly created object is in overlay');
		$obj->banish_object();
	}

	function testDeleteThenCreate()
	{
		$obj = new host_nacoma_Model();
		$obj->get_default_object();
		$obj->obj['host_name'] = 'temp object';
		$obj->obj['alias'] = 'temp object';
		$obj->obj['address'] = 'no address';
		$obj->id = 'new';
		$obj->save_object();
		$this->assertEmpty($obj->validation_error_list, 'User can save new host');
		$obj->banish_object();
		$this->assertEmpty($obj->validation_error_list, 'User can delete new host');
		$obj = new host_nacoma_Model();
		$obj->get_default_object();
		$obj->obj['host_name'] = 'temp object';
		$obj->obj['alias'] = 'temp object';
		$obj->obj['address'] = 'no address';
		$obj->id = 'new';
		$obj->save_object();
		$this->assertEmpty($obj->validation_error_list, 'User can save new host');
		$changelog = new ChangeLog_nacoma_Model();
		$_1 = null; $_2 = null;
		$changelog->undo_others_changes($_1, $_2);
		$obj->banish_object();
	}

	function testAddRelationToNew()
	{
		set_user('host_7', array('op5_admins'));
		$obj = new host_nacoma_Model('host_7');

		$newhost = new host_nacoma_Model();
		$newhost->get_default_object();
		$newhost->obj['host_name'] = 'temp host';
		$newhost->obj['alias'] = 'temp host';
		$newhost->obj['address'] = 'no address';
		$newhost->save_object();
		$this->assertEmpty($newhost->validation_error_list, 'Save new user successful');

		$obj->obj['parents'][] = $newhost->id;
		$obj->obj['notes'] = 'Foohoo!';
		$obj->save_object();

		$changes = new ChangeLog_nacoma_Model();
		$_1 = null; $_2 = null;
		$changes->undo_others_changes($_1, $_2);

		$newhost->banish_object();
	}

	function testMoveToChangelogHistory()
	{
		set_user('host_7', array('op5_admins'));
		$res = nacoma_db::exec_query('SELECT count(1) FROM changelog_history');
		$ary = $res->fetchAll(PDO::FETCH_NUM);
		$orig_count = $ary[0][0];
		unset($res);

		$obj = new host_nacoma_Model('host_42');
		$this->assertTrue(!empty($obj->obj), "Object exists");
		$obj->obj['notes'] = 'foobar';
		$obj->save_object();
		$this->assertEmpty($obj->validation_error_list, 'Object was saved');

		$res = nacoma_db::exec_query('SELECT count(1) FROM changelog_history');
		$ary = $res->fetchAll(PDO::FETCH_NUM);
		$count = $ary[0][0];
		unset($res);
		$this->assertEquals($orig_count, $count, 'Changelog history not changed when object is saved');

		$changes = new ChangeLog_nacoma_Model();
		$changes->archive_changes();
		$res = nacoma_db::exec_query('SELECT count(1) FROM changelog_history');
		$ary = $res->fetchAll(PDO::FETCH_NUM);
		$count = $ary[0][0];
		unset($res);
		$this->assertEquals($orig_count+1, $count, 'Changelog history gets new change appended to it');

		$res = nacoma_db::exec_query('SELECT * FROM changelog_history ORDER BY id DESC LIMIT 1');
		$ch_obj = $res->fetch(PDO::FETCH_ASSOC);
		unset($res);
		$res = nacoma_db::exec_query('SELECT * FROM changelog ORDER BY id DESC LIMIT 1');
		$c_obj = $res->fetch(PDO::FETCH_ASSOC);
		unset($res);
		foreach (array('object_type', 'oldname', 'newname', 'username', 'time', 'oldstate', 'newstate') as $field)
			$this->assertEquals($ch_obj[$field], $c_obj[$field], 'Changelog history has same '.$field);
		$this->assertEquals($obj->obj['host_name'], $ch_obj['object_name'], "Changelog history has object's name");

		nacoma_db::exec_query('DELETE FROM changelog_history WHERE id = '.$ch_obj['id']);
	}

	# test for bug #4541
	function testObjectIsCollapsed()
	{
		$tmpl = new host_template_nacoma_Model();
		$tmpl->get_default_object();
		$tmpl->obj['name'] = 'test_template';
		$tmpl->obj['contact_groups'] = array(1);
		$tmpl->obj['check_command'] = 1;
		$tmpl->obj['notification_period'] = 1;
		$tmpl->save_object();
		$this->assertEmpty($tmpl->validation_error_list, "User can save new template");
		$obj = new host_nacoma_Model();
		$obj->get_default_object();
		$obj->obj['host_name'] = 'test_object';
		$obj->obj['alias'] = 'test_object';
		$obj->obj['address'] = 'localhost';
		$obj->obj['template'] = $tmpl->id;
		$obj->expand_object();
		$obj->save_object();
		$this->assertEmpty($obj->validation_error_list, 'User can save new host');

		$saved_obj = new host_nacoma_Model($obj->id);
		$this->assertTrue(!isset($saved_obj->obj['contact_groups']), "Contact group only in template");
		$saved_obj->expand_object();
		$this->assertEquals(array_values($saved_obj->obj['contact_groups']), array_values($tmpl->obj['contact_groups']), "Template's contact group is present in object after expansion");
		$saved_obj->collapse_object();
		$this->assertTrue(!isset($saved_obj->obj['contact_groups']), "Template's contact group is removed after collapsing");
		$saved_obj->expand_object();
		$this->assertEquals(array_values($saved_obj->obj['contact_groups']), array_values($tmpl->obj['contact_groups']), "Template's contact group is present in object after reexpansion");
		$saved_obj->obj['contact_groups'] = $tmpl->obj['contact_groups'];
		$saved_obj->collapse_object();
		$this->assertTrue(!isset($saved_obj->obj['contact_groups']), "Template's contact group is removed after recollapsing");
	}

	function testReferencesToPending() {
		$host = new host_nacoma_Model();
		$host->get_default_object();
		$host->obj['host_name'] = 'testhost';
		$host->obj['alias'] = 'testhost';
		$host->obj['address'] = 'testhost';
		$host->save_object();
		$this->assertEmpty($host->validation_error_list, 'Save new host');
		$this->assertTrue(is_object_pending($host->type, $host->id), 'New host is pending');

		$host2 = new host_nacoma_Model('host_42');
		$this->assertTrue(!empty($host2->obj), 'host exists');
		$this->assertTrue(!is_object_pending($host2->type, $host2->id), 'Old host is not pending');
		$orig_parents = $host2->obj['parents'];
		$host2->obj['parents'][] = $host->id;
		$host2->obj['notes'] = 'this is a brand new note';
		$host2->save_object();
		$this->assertEmpty($host2->validation_error_list, 'Add pending rel');
		$host2->obj['parents'] = $orig_parents;
		$host2->obj['notes'] = 'this is a completely different note';
		$host2->save_object();
		$this->assertEmpty($host2->validation_error_list, 'Remove pending rel');

		$svc = new service_nacoma_Model(224);
		$this->assertTrue(!empty($svc->obj), 'service exists');
		$this->assertTrue(!is_object_pending($svc->type, $svc->id), 'Old service is not pending');
		$orig_parent = $svc->obj['host_name'];
		$svc->obj['host_name'] = $host->id;
		$svc->master_obj_id = $host->id;
		$svc->obj['notes'] = 'this is a brand new note';
		$svc->save_object();
		$this->assertEmpty($svc->validation_error_list, 'Add relation to pending');
		$svc->obj['host_name'] = $orig_parent;
		$svc->master_obj_id = $orig_parent;
		$svc->obj['notes'] = 'this is a completely different note';
		$svc->save_object();
		$this->assertEmpty($svc->validation_error_list, 'Remove relation to pending');

		$host->banish_object();
		$res = nacoma_db::exec_query('SELECT COUNT(1) FROM changelog');
		$row = $res->fetch(PDO::FETCH_NUM);
		unset($res);
		$this->assertEquals($row[0], 10, 'There are 10 changes');
	}

	function testSummary() {
		// first, hack: ignore that we're magically deleting a hostextinfo in the background
		$host = new host_nacoma_Model(46);
		$host->save_object();
		$changes = new ChangeLog_nacoma_Model();
		$changes->delete_changes();

		$changes = new ChangeLog_nacoma_Model();
		$this->assertTrue(strpos($changes->get_summary(), 'no changes') !== false, 'Nice summary when no changes');
		$this->assertEmpty($changes->get_users(), 'Empty user list');
		$this->assertTrue(strpos($changes->get_text_list(), "No changes\n") !== false, 'Nice detailed list when no changes');

		$host = new host_nacoma_Model(46);
		$host->obj['notes'] = 'test';
		$host->save_object();
		$changes = new ChangeLog_nacoma_Model();
		$this->assertTrue(strpos($changes->get_summary(), '1 host') !== false, 'Not empty summary after change');
		$this->assertEmpty($changes->get_other_users(), 'No other users, so get_other_users empty');
		$this->assertEquals($changes->get_users(), array('host_7'), 'get_users not empty after change');
		$this->assertTrue(strpos($changes->get_text_list(), "Added attribute") !== false, 'Attribute change presented nicely with one change');
		$this->assertTrue(strpos($changes->get_text_list(), "Changes in host") !== false, 'Host change presented nicely with one change');

		$host->obj['notes'] = 'test2';
		$host->save_object();
		$changes = new ChangeLog_nacoma_Model();
		$this->assertTrue(strpos($changes->get_summary(), '1 host') !== false, 'The two changes are merged');
		$this->assertEmpty($changes->get_other_users(), 'No other users, so get_other_users empty');
		$this->assertEquals($changes->get_users(), array('host_7'), 'get_users not empty after change');
		$this->assertTrue(strpos($changes->get_text_list(), "Added attribute") !== false, 'First attribute change still presented with two changes');
		$this->assertTrue(strpos($changes->get_text_list(), "Changed attribute") !== false, 'Extra attribute change presented as well');
		$this->assertTrue(substr_count($changes->get_text_list(), "Changes in host") === 1, 'The two changes are visually merged');

		$changes->delete_changes();
		$changes = new ChangeLog_nacoma_Model();
		$this->assertTrue(strpos($changes->get_summary(), 'no changes') !== false, 'Nice summary when changes deleted again');
		$this->assertEmpty($changes->get_users(), 'Empty user list when changes deleted again');
		$this->assertTrue(strpos($changes->get_text_list(), "No changes\n") !== false, 'Nice detailed list when changes deleted again');
	}

	function testRenames() {
		$host = new host_nacoma_Model('host_2');
		$host->save_object();
		$changes = new ChangeLog_nacoma_Model();
		$changes->delete_changes();

		$service = null;
		$res = nacoma_db::exec_query("SELECT id FROM service WHERE host_name = $host->id");
		foreach ($res->fetchAll(PDO::FETCH_ASSOC) as $id) {
			$service = new service_nacoma_Model($id['id']);
			if ($host->obj['file_id'] != $service->obj['file_id'])
				break;
		}
		$host->obj['host_name'] = 'woha';
		$this->assertTrue($host->save_object(), 'Save changed host');
		$changes = new ChangeLog_nacoma_Model();
		$overlay = null; $changed_files = null;
		$changes->undo_others_changes($overlay, $changed_files);
		$this->assertEquals(count($changed_files), 2, "Slave object's file must be reexported");
	}

	// bugs #4994, #5041
	function testChangeNonUniqueRelations() {
		set_user('host_7', array('op5_admins'));
		$host = new host_nacoma_Model('host_9');
		$new_host = $host->clone_object();
		$new_host->obj['parents'] = array($host->id);
		$new_host->obj['host_name'] = 'new_host_name';
		$new_host->save_object();
		$this->assertEmpty($new_host->validation_error_list, "Copied host saved");

		$changes = new ChangeLog_nacoma_Model();
		$overlay = false; $files = false;
		$changes->undo_others_changes($overlay, $files);
		$exported_obj = $overlay['host'][$new_host->id];
		$this->assertEquals($exported_obj->obj['parents'], array_unique($exported_obj->obj['parents']), 'There are no duplicated values in parents');
	}

	function testDifferentUsersDifferentFiles() {
		// host_9 is in testconfig
		// host_25 is in mangleconfig
		set_user('host_7', array('op5_admins'));

		$host = new host_nacoma_Model('host_25');
		$host->obj['notes'] = 'a note';
		$this->assertTrue($host->save_object(), 'Admin can save host change');

		set_user('contact_limited', array('contact_limited'));
		$contact = new contact_nacoma_Model('contact1');
		$contact->obj['alias'] = 'an alias';
		$this->assertTrue($contact->save_object(), 'Limited can save contact change');

		$changes = new ChangeLog_nacoma_Model();
		$overlay = false; $files = false;
		$changes->undo_others_changes($overlay, $files);
		$this->assertCount(1, $files, 'Only one file should have included changes: '.var_export($files, true));
		$this->assertEmpty($overlay['host'], 'No host in overlay');
		$this->assertCount(1, $overlay['contact'], 'One contact in overlay');
	}

	// bug #5980
	function testUnsetBool() {
		set_user('host_7', array('op5_admins'));
		// first, hack: ignore that we're magically deleting a hostextinfo in the background
		$host = new host_nacoma_Model('host_24');
		$host->save_object();
		$changes = new ChangeLog_nacoma_Model();
		$changes->delete_changes();

		$host = new host_nacoma_Model('host_24');
		$host->obj['passive_checks_enabled'] = '0';
		$this->assertTrue($host->save_object(), 'Object was saved after disabling passive checks');
		$this->assertEquals(Changelog_nacoma_Model::get_changes_count(), 1, 'There is one change after saving');
		$changes = new ChangeLog_nacoma_Model();
		$overlay = false; $files = false;
		$changes->undo_others_changes($overlay, $files);
		$this->assertTrue($overlay != false, 'There is an overlay');
	}
}
