<?php

class SpecialConfigs extends PHPUnit_Framework_TestCase
{
	protected $backupGlobalsBlacklist = array('master_ary', 'Supported_Object_Types');
	function tearDown()
	{
		evil_global_teardown();
	}

	/**
	 * What is that I smell? Is it code?
	 */
	function cleanup_globals()
	{
		global $slave_obj_array;
		global $import_warnings, $import_errors, $index, $num_objs, $obj_array;
		global $master_list;
		global $custom_vars, $Template_Cache;
		$master_list = false;
		$slave_obj_array = array();
		$import_warnings = array();
		$import_errors = array();
		$index = array();
		$num_objs = array();
		$obj_array = array();
		$custom_vars = array();
		$Template_Cache = array();
		naemon_nacoma_Model::$objects_for_user = array();
	}

	function setUp()
	{
		evil_global_setup();
		set_user('host_7', array('op5_admins'));
		require_once('include/data.inc.php');
		$this->cleanup_globals();
		nacoma_db::exec_query('DELETE FROM objlocks');
		nacoma_db::exec_query('DELETE FROM changelog');
		nacoma_db::exec_query('DELETE FROM action_log');
	}

	function config_equal($pre_fname, $post_fname)
	{
		$empty_file = array("###############################################################################\n",
		                    "#  Generated by op5 Monitor webconfiguration exporter\n",
		                    "#\n",
		                    "# Exported <date> by <user>\n",
		                    "#\n\n");
		if ($pre_fname === null)
			$pre = $empty_file;
		else
			$pre = file($pre_fname);
		if ($post_fname === null)
			$post = $empty_file;
		else
			$post = file($post_fname);
		# line 4 is the one with the export date. Unset:
		if (isset($pre[3]))
			unset($pre[3]);
		if (isset($post[3]))
			unset($post[3]);
		$pre = implode("", $pre);
		$post = implode("", $post);
		$this->assertEquals($pre, $post, "Unexpected contents of config file $post_fname");
	}

	function testExportAfterDelete()
	{
		global $HOME;
		$empty_pre = $HOME.'deleted-dependencies.cfg';
		$hg_dep_pre = $HOME.'related-dependencies-hg.cfg';
		$host_dep_pre = $HOME.'related-dependencies-host.cfg';
		$pre_cfg = array($empty_pre, $hg_dep_pre, $host_dep_pre);
		$post_cfg = array();
		foreach ($pre_cfg as $cfg) {
			$post_cfg[] = $cfg.'.new';
		}
		$this->assertTrue(
			import_config('phptap', $pre_cfg),
			"import_config('phptap', '" . print_r($pre_cfg, true) . "')"
		);
		$host = new host_nacoma_Model('the_host');
		$this->assertTrue($host->id && is_numeric($host->id), 'Host was found');
		$host->banish_object();
		$cg = new contactgroup_nacoma_Model('the_contactgroup');
		$this->assertTrue($cg->id && is_numeric($cg->id), 'Contactgroup was found');
		$cg->banish_object();

		foreach ($post_cfg as $cfg)
			touch($cfg);

		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		nacoma_db::exec_query($query);

		$changes = new ChangeLog_nacoma_Model();
		$ret = do_export($changes);
		$this->config_equal($empty_pre.'.new', null);
		$this->config_equal($hg_dep_pre.'.new', $hg_dep_pre.'.expected');
		$this->config_equal($host_dep_pre.'.new', $host_dep_pre.'.expected');
	}

	function testExportAfterOtherCreate()
	{
		global $HOME;
		$empty_pre = $HOME.'deleted-dependencies.cfg';
		$hg_dep_pre = $HOME.'related-dependencies-hg.cfg';
		$host_dep_pre = $HOME.'related-dependencies-host.cfg';
		$pre_cfg = array($hg_dep_pre, $host_dep_pre, $empty_pre);
		$post_cfg = array();
		foreach ($pre_cfg as $cfg) {
			$post_cfg[] = $cfg.'.new';
		}
		$this->assertTrue(
			import_config('phptap', $pre_cfg),
			"import_config('phptap', '" . print_r($pre_cfg, true) . "')"
		);
		set_user('the_contact', array('op5_admins'));
		$this->cleanup_globals();
		$h = new host_nacoma_Model();
		$h->get_default_object();
		$h->normalize(array(
			'host_name' => 'unexported_host',
			'max_check_attempts' => 1,
			'notification_interval' => 1,
			'check_period' => 0,
			'notification_period' => '24x7',
		));
		$this->assertTrue($h->save_object(), 'host should be saved, got '.var_export($h->validation_error_list, true));
		$svc = new service_nacoma_Model();
		$svc->normalize(array(
			'host_name' => 'unexported_host',
			'service_description' => 'not_exported',
			'template' => 'default-service',
			'max_check_attempts' => 1,
			'notification_interval' => 1,
			'check_period' => '24x7',
			'check_command' => 'check-host-alive',
			'check_interval' => 1,
			'retry_interval' => 1,
			'notification_period' => '24x7',
		));
		$this->assertTrue($svc->save_object(), 'service should be saved, got '.var_export($svc->validation_error_list, true));
		set_user('host_7', array('op5_admins'));
		$this->cleanup_globals();
		$hg = new hostgroup_nacoma_Model('the_hostgroup');
		$hg->obj['notes'] = 'foo';
		$this->assertTrue($hg->save_object(), 'hostgroup should be savable, got '.var_export($hg->validation_error_list, true));
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		nacoma_db::exec_query($query);

		$chng = new ChangeLog_nacoma_Model();
		$ret = do_export($chng);
		$this->assertSame($ret, true, 'Config export was successful');
		$this->config_equal($hg_dep_pre.'.new', $hg_dep_pre.'.expected2');
	}

	/**
	 * Test for bug #4999
	 */
	function testTrailingComma()
	{
		global $HOME;
		$pre = $HOME.'trailing-comma.cfg';
		$post = $HOME.'trailing-comma.cfg.expected';
		$this->assertTrue(import_config('phptap', $pre), "import_config('phptap', '$pre')");

		touch($pre.'.new');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		nacoma_db::exec_query($query);

		$ret = do_export(true);
		$this->assertSame($ret, true, 'Config export was successful');
		$this->config_equal($pre.'.new', $post);
	}

	/**
	 * Test for bug #5105
	 */
	function testMinimizeConfig()
	{
		global $HOME;
		$pre = $HOME.'minimize-template.cfg';
		$post = $HOME.'minimize-template.cfg.expected';
		$this->assertTrue(import_config('phptap', $pre), "import_config('phptap', '$pre')");

		touch($pre.'.new');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		nacoma_db::exec_query($query);

		$host = new host_nacoma_Model('test');
		$this->assertTrue($host->save_object(), 'Save object is ok');

		$changes = new ChangeLog_nacoma_Model();
		$ret = do_export($changes);
		$this->config_equal($pre.'.new', $post);
	}

	/**
	 * The bug is that get_object can't fetch any more relations than it fetches objects
	 */
	function testChangeUnrelated()
	{
		global $HOME;
		$pre_base = $HOME.'hostwithservices.cfg';
		$pre_deps = $HOME.'separate_service.cfg';
		$this->assertTrue(import_config('phptap', array($pre_base, $pre_deps)), "import_config('phptap', array($pre_base, $pre_deps))");
		global $IMPORTING;
		$IMPORTING=false;

		touch($pre_base.'.new');
		touch($pre_deps.'.new');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		nacoma_db::exec_query($query);

		$svc = new service_nacoma_Model('base_host;separate_service');
		$svc->obj['notes'] = 'foo';
		$this->assertTrue($svc->save_object(), 'Save object is ok');
		$this->assertEmpty($svc->validation_error_list, 'No save failures');
		unset($svc->obj['notes']);
		$this->assertTrue($svc->save_object(), 'Save object is ok');
		$this->assertEmpty($svc->validation_error_list, 'No save failures');

		$changes = new ChangeLog_nacoma_Model();
		$ret = do_export($changes);
		$this->assertSame(true, $ret, "Export was successful");
		$this->config_equal($pre_deps.'.new', $pre_deps);
	}

	function testChangeChild()
	{
		global $HOME;
		$pre_first = $HOME.'emptyhostinafile1.cfg';
		$pre_second = $HOME.'emptyhostinafile2.cfg';
		$this->assertTrue(import_config('phptap', array($pre_first, $pre_second)), "import_config('phptap', array($pre_first, $pre_second))");
		global $IMPORTING;
		$IMPORTING=false;

		touch($pre_first.'.new');
		touch($pre_second.'.new');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		nacoma_db::exec_query($query);

		$host1 = new host_nacoma_Model('hostinfile1');
		$this->assertTrue(!empty($host1->obj), 'host has data');

		$host2 = new host_nacoma_Model('hostinfile2');
		$this->assertTrue(!empty($host2->obj), 'host has data');

		$host2->obj['parents'] = array($host1->id);
		$this->assertTrue($host2->save_object(), 'host with parents saved');
		$this->assertEmpty($host2->validation_error_list, 'Error list was empty');
		$host2 = new host_nacoma_Model('hostinfile2');
		$this->assertEquals($host2->obj['parents'], array($host1->id), 'Parent is still there after saving');

		$changes = new ChangeLog_nacoma_Model();
		$ret = do_export($changes);
		$this->assertSame(true, $ret, "Export was successful");
		$host1 = new host_nacoma_Model('hostinfile1');
		$this->assertEquals($host1->obj['children'], array($host2->id), 'Parent has child after exporting');

		$host1->obj['host_name'] = 'hostinfilewithnewname';
		$this->assertTrue($host1->save_object(), 'host with parents saved');
		$this->assertEmpty($host1->validation_error_list, 'Error list was empty');

		$changes = new ChangeLog_nacoma_Model();
		$ret = do_export($changes);
		$this->assertSame(true, $ret, "Export was successful");
		$this->config_equal($pre_first.'.expected', $pre_first.'.new');
		$this->config_equal($pre_second.'.expected', $pre_second.'.new');
	}

	/**
	 * Test for bug #7452
	 */
	function testAddContactGroupMember()
	{
		global $HOME;
		$pre_first = $HOME.'testAddContactGroupMember-cg.cfg';
		$pre_second = $HOME.'testAddContactGroupMember-c.cfg';

		$this->assertTrue(import_config('phptap', array($pre_first, $pre_second)), "import_config('phptap', array($pre_first, $pre_second))");
		global $IMPORTING;
		$IMPORTING=false;

		touch($pre_first.'.new');
		touch($pre_second.'.new');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		nacoma_db::exec_query($query);

		$cg = new contactgroup_nacoma_Model('the_contactgroup');
		$this->assertTrue(!empty($cg->obj), 'Contactgroup should be loaded with data');
		$c = new contact_nacoma_Model('the_contact');
		$this->assertTrue(!empty($c->obj), 'Contact should be loaded with data');
		$cg->obj['members'] = array($c->id);

		$this->assertTrue($cg->save_object(), 'Contactgroup with new member should be saved correctly');
		$this->assertEmpty($cg->validation_error_list, 'Contactgroup should have empty error list after saving');

		$changes = new ChangeLog_nacoma_Model();
		$ret = do_export($changes);
		$this->assertSame(true, $ret, "Export was successful");
		$this->config_equal($pre_first.'.new', $pre_first.'.expected');
		$this->config_equal($pre_second.'.new', $pre_second.'.expected');

		$hg = new hostgroup_nacoma_Model('the_hostgroup');
		$this->assertTrue(!empty($hg->obj), 'Hostgroup should be loaded with data');
		$h = new host_nacoma_Model('the_host');
		$this->assertTrue(!empty($h->obj), 'Host should be loaded with data');
		$hg->obj['members'] = array($h->id);

		$this->assertTrue($hg->save_object(), 'Hostgroup with new member should be saved correctly');
		$this->assertEmpty($hg->validation_error_list, 'Hostgroup should have empty error list after saving');

		$changes = new ChangeLog_nacoma_Model();
		$ret = do_export($changes);
		$this->assertSame(true, $ret, "Export was successful");
		$this->config_equal($pre_first.'.new', $pre_first.'.expected2');
		$this->config_equal($pre_second.'.new', $pre_second.'.expected2');
	}

	function testTemplateInTemplate()
	{
		global $HOME;
		$pre = $HOME.'template-in-template.cfg';

		$this->assertTrue(import_config('phptap', array($pre)), "import_config('phptap', array($pre))");
		global $IMPORTING;
		$IMPORTING=false;

		touch($pre.'.new');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		nacoma_db::exec_query($query);

		# customer use case
		$parent = new service_template_nacoma_Model('default-service');
		$svctmpl = new service_template_nacoma_Model();
		$svctmpl->obj['name'] = 'ping';
		$svctmpl->obj['active_checks_enabled'] = 1;
		$svctmpl->obj['check_freshness'] = 1;
		$svctmpl->obj['freshness_threshold'] = 1300;
		$svctmpl->obj['notes'] = 'Ping tjänst - ICMP echo';
		$svctmpl->obj['passive_checks_enabled'] = 1;
		$svctmpl->obj['template'] = $parent->id;

		$this->assertTrue($svctmpl->save_object(), 'New service template with parent template should be saved correctly');
		$this->assertEmpty($svctmpl->validation_error_list, 'New service template should have empty error list after saving');

		# test other templates while we're at it
		$parent = new host_template_nacoma_Model('default-host-template');
		$hosttmpl = new host_template_nacoma_Model();
		$hosttmpl->obj['name'] = 'monitor';
		$hosttmpl->obj['template'] = $parent->id;

		$this->assertTrue($hosttmpl->save_object(), 'New host template with parent template should be saved correctly');
		$this->assertEmpty($hosttmpl->validation_error_list, 'New host template should have empty error list after saving');

		$parent = new contact_template_nacoma_Model('default-contact');
		$contacttmpl = new contact_template_nacoma_Model();
		$contacttmpl->obj['name'] = 'monitor';
		$contacttmpl->obj['template'] = $parent->id;

		$this->assertTrue($contacttmpl->save_object(), 'New contact template with parent template should be saved correctly');
		$this->assertEmpty($contacttmpl->validation_error_list, 'New contact template should have empty error list after saving');

		# test non-templates using these templates
		$host = new host_nacoma_Model();
		$host->obj['host_name'] = 'test_host';
		$host->obj['template'] = $hosttmpl->id;
		$host->obj['notification_period'] = 1;
		$this->assertTrue($host->save_object(), 'New host with template should be saved correctly');
		$this->assertEmpty($host->validation_error_list, 'New host should have empty error list after saving');

		$svc = new service_nacoma_Model();
		$svc->obj['host_name'] = $host->id;
		$svc->obj['service_description'] = 'test_description';
		$svc->obj['template'] = $svctmpl->id;
		$svc->obj['notification_period'] = 1;
		$svc->obj['check_period'] = 1;
		$svc->obj['check_command'] = 1;
		$this->assertTrue($svc->save_object(), 'New service with template should be saved correctly');
		$this->assertEmpty($svc->validation_error_list, 'New service should have empty error list after saving');

		$contact = new contact_nacoma_Model();
		$contact->get_default_object();
		$contact->obj['contact_name'] = 'test_contact';
		$contact->obj['alias'] = 'test_contact';
		$contact->obj['template'] = $contacttmpl->id;
		$this->assertTrue($contact->save_object(), 'New contact with template should be saved correctly');
		$this->assertEmpty($contact->validation_error_list, 'New contact should have empty error list after saving');

		$changes = new ChangeLog_nacoma_Model();
		$ret = do_export($changes);
		$this->assertSame(true, $ret, "Export was successful");
		$this->config_equal($pre.'.expected', $pre.'.new');
	}

	function testPervertedAccess()
	{
		global $HOME;
		$pre = $HOME.'objecttree.cfg';

		$this->assertTrue(import_config('phptap', array($pre)), "import_config('phptap', array($pre))");
		global $IMPORTING;
		$IMPORTING=false;

		$ug = new usergroup_nacoma_Model('all_by_contact');
		$ug->obj['access_rights'] = array(
			'configuration_information',
			'host_view_contact',
			'host_template_view_all',
			'hostgroup_view_contact',
			'service_view_contact',
			'service_template_view_all',
			'servicegroup_view_contact',
			'contact_view_contact',
			'contact_template_view_all',
			'contactgroup_view_contact',
		);
		$this->assertTrue($ug->save_object(), 'New usergroup should be saved, but got: '.var_export($ug->validation_error_list, true));

		set_user('the_contact', array('all_by_contact'));
		/* I changed user, let's flush caches */
		$this->cleanup_globals();
		$objects = array(
			'host' => array(
				'branch1_host',
				'branch2_host',
				'branch4_host',
			),
			'host_template' => array(
				'branch1_htmpl',
				'branch2_htmpl',
				'branch4_htmpl',
			),
			'hostgroup' => array(
				'branch2_hg',
				'branch4_hgroup2',
				'branch4_hgroup1',
			),
			'service' => array(
				'branch3_host;branch3_svc',
				'branch1_host;branch1_svc',
				'branch2_host;branch2_svc',
				'branch4_svc_host;branch4_svc',
			),
			'service_template' => array(
				'branch3_stmpl1',
				'branch3_stmpl2',
				'branch4_stmpl',
			),
			'servicegroup' => array(
				'branch1_sg1',
				'branch1_sg2',
				'branch3_sgroup',
			),
			'contact' => array(
				'the_contact',
				'branch2_contact',
			),
			'contact_template' => array(
				'branch4_ctmpl1',
				'branch4_ctmpl2',
			),
			'contactgroup' => array(
				'parent_cgroup',
				'grandparent_cgroup',
				'branch2_cgroup',
			),
		);
		foreach ($objects as $type => $objs) {
			$class = $type.'_nacoma_Model';
			foreach ($objs as $objname) {
				$obj = new $class($objname);
				$this->assertTrue($obj->can_user_read($obj->id), "$type $objname should load properly");
			}
			$obj = new $class('uncon_'.$type);
			$this->assertTrue(!$obj->id || !$obj->can_user_read($obj->id), "Unconnected $type should not load properly");
		}
	}

	/**
	 * Test for bug #8530
	 */
	function test_new_host_and_service_dep_at_once()
	{
		global $HOME;
		$pre = $HOME.'test_new_host_and_service_dep_at_once.cfg';
		$this->assertTrue(import_config('phptap', array($pre)), "import_config('phptap', array($pre))");
		global $IMPORTING;
		$IMPORTING=false;

		touch($pre.'.new');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		nacoma_db::exec_query($query);

		$hst = new host_nacoma_Model();
		$hst->get_default_object();
		$hst->obj['host_name'] = 'new_host';
		$hst->obj['max_check_attempts'] = 1;
		$hst->obj['notification_interval'] = 1;
		$hst->obj['notification_period'] = 1;
		$this->assertTrue($hst->save_object(), 'saving new host should work, got ' . var_export($hst->validation_error_list, true));

		$svc = new service_nacoma_Model();
		$svc->get_default_object();
		$svc->obj['host_name'] = $hst->id;
		$svc->obj['service_description'] = 'new_service';
		$svc->obj['check_command'] = 1;
		$this->assertTrue($svc->save_object(), 'saving new service should work, got ' . var_export($svc->validation_error_list, true));

		$other_svc = new service_nacoma_Model('base_host;snuggly_service');
		$sd = new servicedependency_nacoma_Model();
		$sd->get_default_object();
		$sd->obj['dependent_service'] = $svc->id;
		$sd->obj['service'] = $other_svc->id;
		$this->assertTrue($sd->save_object(), 'saving new servicedpendendency should work, got ' . var_export($sd->validation_error_list, true));

		$changes = new ChangeLog_nacoma_Model();
		$ret = do_export($changes);
		$this->assertSame(true, $ret, "Export was successful");
		$this->config_equal($HOME.'test_new_host_and_service_dep_at_once.cfg.expected', $pre.'.new');
	}
}
